@@ -80,9 +80,10 @@ class ImageDllCharacteristics(Enum):
     NO_ISOLATION = 0x0200
     NO_SEH = 0x0400
     NO_BIND = 0x0800
+    APP_CONTAINER = 0x1000
     WDM_DRIVER = 0x2000
+    CONTROL_FLOW_GUARD = 0x4000
     TERMINAL_SERVER_AWARE = 0x8000
-    CONTROL_FLOW_GUARD = 0xc000
 
 
 class ImageDirectoryEntry(Enum):
@@ -188,6 +189,10 @@ class IMAGE_EXPORT_DIRECTORY(Structure):
                 ('AddressOfNameOrdinals',c_uint)
                 ]
 
+class GUARD_CFF_ENTRY(Structure):
+    _fields_ = [('rva',c_uint),
+                ('flag', c_byte)]
+
 ##################### PE32 ########################
 
 class IMAGE_OPTIONAL_HEADER(Structure):
@@ -232,6 +237,35 @@ class PE32_IMAGE_NT_HEADERS(Structure):
 class PE32(object):
     IMAGE_NT_HEADERS = PE32_IMAGE_NT_HEADERS
 
+
+class IMAGE_LOAD_CONFIG_DIRECTORY32(Structure):
+    _fields_ = [('Size', c_uint),
+                ('TimeDateStamp', c_uint),
+                ('MajorVersion', c_ushort),
+                ('MinorVersion', c_ushort),
+                ('GlobalFlagsClear', c_uint),
+                ('GlobalFlagsSet', c_uint),
+                ('CriticalSectionDefaultTimeout', c_uint),
+                ('DeCommitFreeBLockThreshold', c_uint),
+                ('DeCommitTotalFreeThreshold', c_uint),
+                ('LockPrefixTable', c_uint),
+                ('MaximumAllocationSize', c_uint),
+                ('VirtualMemoryThreshold', c_uint),
+                ('ProcessHeapFlags', c_uint),
+                ('ProcessAffinityMask', c_uint),
+                ('CSDVersion', c_ushort),
+                ('Reserved1', c_ushort),
+                ('EditList', c_uint),
+                ('SecurityCookie', c_uint),
+                ('SEHandlerTable', c_uint),
+                ('SEHandlerCount', c_uint),
+                ('GuardCFCheckFunctionPointer', c_uint),
+                ('Reserved2', c_uint),
+                ('GuardCFFunctionTable', c_uint),
+                ('GuardCFFunctionCount', c_uint),
+                ('GuardFlags', c_uint)]
+
+
 ######################### PE64 ########################
 
 class IMAGE_OPTIONAL_HEADER_PE32_PLUS(Structure):
@@ -275,6 +309,34 @@ class PE64_IMAGE_NT_HEADERS(Structure):
 class PE64(object):
     IMAGE_NT_HEADERS = PE64_IMAGE_NT_HEADERS
 
+
+class IMAGE_LOAD_CONFIG_DIRECTORY64(Structure):
+    _fields_ = [('Size', c_uint),
+                ('TimeDateStamp', c_uint),
+                ('MajorVersion', c_ushort),
+                ('MinorVersion', c_ushort),
+                ('GlobalFlagsClear', c_uint),
+                ('GlobalFlagsSet', c_uint),
+                ('CriticalSectionDefaultTimeout', c_uint),
+                ('DeCommitFreeBLockThreshold', c_ulonglong),
+                ('DeCommitTotalFreeThreshold', c_ulonglong),
+                ('LockPrefixTable', c_ulonglong),
+                ('MaximumAllocationSize', c_ulonglong),
+                ('VirtualMemoryThreshold', c_ulonglong),
+                ('ProcessAffinityMask', c_ulonglong),
+                ('ProcessHeapFlags', c_uint),
+                ('CSDVersion', c_ushort),
+                ('Reserved1', c_ushort),
+                ('EditList', c_ulonglong),
+                ('SecurityCookie', c_ulonglong),
+                ('SEHandlerTable', c_ulonglong),
+                ('SEHandlerCount', c_ulonglong),
+                ('GuardCFCheckFunctionPointer', c_ulonglong),
+                ('Reserved2', c_ulonglong),
+                ('GuardCFFunctionTable', c_ulonglong),
+                ('GuardCFFunctionCount', c_ulonglong),
+                ('GuardFlags', c_uint)]
+
 ##################### Container ###################
 
 
@@ -338,6 +400,12 @@ class ExportDirectoryData(Container):
     functions = list of FunctionData
     """
 
+class LoadConfigData(Container):
+    """"
+    header = IMAGE_LOAD_CONFIG_DIRECTORY32/IMAGE_LOAD_CONFIG_DIRECTORY64
+    cfGuardedFunctions = list of relative virtual addresses (RVA) of cfg allowed call/jmp targets. Empty if CFG not supported
+    """
+
 class FunctionData(Container):
     """
     name = name of the function (str)
@@ -467,6 +535,12 @@ class PE(Binary):
         import_data_directory_data = self._parseDataDirectoryImport(import_data_directory, import_section)
         data_directory_data_list[ImageDirectoryEntry.IMPORT] = import_data_directory_data
 
+        # parse DataDirectory[LOAD_CONFIG]
+        loadconfig_data_directory = imageNtHeaders.header.OptionalHeader.DataDirectory[ImageDirectoryEntry.LOAD_CONFIG]
+        loadconfig_section = self._getSectionForDataDirectoryEntry(loadconfig_data_directory, sections)
+        loadconfig_data = self._parseLoadConfig(loadconfig_data_directory, loadconfig_section)
+        data_directory_data_list[ImageDirectoryEntry.LOAD_CONFIG] = loadconfig_data
+
         return data_directory_data_list
 
     def _parseDataDirectoryExport(self, data, dataDirectoryEntry, exportSection):
@@ -506,7 +580,7 @@ class PE(Binary):
         import_descriptors = []
         while True:
             import_descriptor = IMAGE_IMPORT_DESCRIPTOR.from_buffer(raw_bytes, offset)
-            offset += sizeof(IMAGE_IMPORT_DESCRIPTOR)
+            
 
             if import_descriptor.OriginalFirstThunk == 0:
                 break
@@ -518,21 +592,65 @@ class PE(Binary):
                 import_address_table =  self.__parseThunks(import_descriptor.FirstThunk, importSection)
 
                 import_descriptors.append(ImportDescriptorData(header=import_descriptor, dllName=dllName, importNameTable=import_name_table, importAddressTable=import_address_table))
+            offset += sizeof(IMAGE_IMPORT_DESCRIPTOR)
         return import_descriptors
 
+    def _getSectionByRVA(self, va):
+        for section in self.sections:
+            address = section.header.VirtualAddress
+            SizeOfRawData = section.header.SizeOfRawData
+            if  address <= va and va < (address + SizeOfRawData):
+                return section
+
+        return
+
+    def _parseLoadConfig(self, loadConfigEntry, loadconfigSection):
+        if not loadconfigSection:
+            return
+            
+        if self._classes == PE64:
+            load_config_directory = IMAGE_LOAD_CONFIG_DIRECTORY64.from_buffer(
+                loadconfigSection.raw, to_offset(loadConfigEntry.VirtualAddress, loadconfigSection))
+
+            pass
+
+        elif self._classes == PE32:
+            load_config_directory = IMAGE_LOAD_CONFIG_DIRECTORY32.from_buffer(
+                loadconfigSection.raw, to_offset(loadConfigEntry.VirtualAddress, loadconfigSection))
+
+            pass
+        else:
+            pass
+
+        guardCFTableRVA = load_config_directory.GuardCFFunctionTable - self.imageBase
+        section = self._getSectionByRVA(guardCFTableRVA)
+        CfGuardedFunctions = set()
+        if section:
+            sectionOffset = guardCFTableRVA - section.header.VirtualAddress
+
+            # loop through the ControlFlow Guard Function table
+            for i in range(0, load_config_directory.GuardCFFunctionCount):
+                cffEntry = GUARD_CFF_ENTRY.from_buffer(section.raw, sectionOffset)
+                CfGuardedFunctions.add(cffEntry.rva)
+                sectionOffset += 5
+
+        return LoadConfigData(header=load_config_directory, cfGuardedFunctions=CfGuardedFunctions )
+
     def __parseThunks(self, thunkRVA, importSection):
         """Parses the thunks and returns a list"""
         offset = to_offset(thunkRVA, importSection)
+        table_offset = 0
         thunks = []
         while True:
             thunk = IMAGE_THUNK_DATA.from_buffer(importSection.raw, offset)
             offset += sizeof(IMAGE_THUNK_DATA)
             if thunk.Ordinal == 0:
                 break
-            thunkData = ThunkData(header=thunk, rva=offset+importSection.header.VirtualAddress, ordinal=None, importByName=None)
+            thunkData = ThunkData(header=thunk, rva=table_offset+thunkRVA,ordinal=None, importByName=None)
             if to_offset(thunk.AddressOfData, importSection) > 0 and to_offset(thunk.AddressOfData, importSection) < len(self._bytes):
                 self.__parseThunkData(thunkData, importSection)
             thunks.append(thunkData)
+            table_offset += 4
         return thunks
 
     def __parseThunkData(self, thunk,importSection):
